﻿using BeatLyrics.Common.Models;
using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq.Expressions;
using System.Reflection;
using BeatLyrics.Tool.Models;

namespace BeatLyrics.Tool.Utils
{
    internal static class JapanesePhonetic
    {
        private static readonly Type ClassType;
        private static readonly Func<object, string> GetDisplayText;
        private static readonly Func<object, string> GetYomiText;

        static JapanesePhonetic()
        {
            ClassType = Type.GetType("Windows.Globalization.JapanesePhoneticAnalyzer," +
                                         " Windows.Globalization, Version=255.255.255.255," +
                                         " Culture=neutral, PublicKeyToken=null, ContentType=WindowsRuntime");
            var itemType = Type.GetType("Windows.Globalization.JapanesePhoneme," +
                                        " Windows.Globalization, Version=255.255.255.255," +
                                        " Culture=neutral, PublicKeyToken=null, ContentType=WindowsRuntime");
            var pArg = Expression.Parameter(typeof(object));
            var pArgConv = Expression.Convert(pArg, itemType);
            GetDisplayText = Expression.Lambda<Func<object, string>>(Expression.Property(pArgConv, "DisplayText"), pArg).Compile();
            GetYomiText = Expression.Lambda<Func<object, string>>(Expression.Property(pArgConv, "YomiText"), pArg).Compile();
        }

        public static TextTuple[] GetWords(string text)
        {
            var ret = (IEnumerable)ClassType.InvokeMember("GetWords", BindingFlags.InvokeMethod, null, null,
                          new object[] { text });

            var lst = new List<TextTuple>();

            foreach (var item in ret)
            {
                var x = new TextTuple { Text = GetDisplayText(item), Ruby = GetYomiText(item) };
                lst.Add(x);
            }

            var arr = lst.ToArray();
            return arr;
        }
    }
}
